Attribute VB_Name = "CarrierTranslation"
Option Explicit

'Carrier Translation V2.00 June 21, 2016.
'Reads host and Clippership carrier codes from dbf on startup, stores in two collections, one for translating
'from host carrier codes to Clippership, the other for the other direction.
'Have function to edit the database, but don't want to expose it to users unless it is specifically required for a given
'installation. Otherwise, access through cf02. In either case, all systems have to restart CS to see the changes.
'Only supports carrier codes for now - does not support payment terms, accessories, etc.
'Assumes 1 to 1 relationship between host and clippership carrier codes. Can't have duplicates of either.

Public Const defaultHostCarrierCode As String = "SHPC"
Private carToCsDb As String
Private toCSOOSCarrierCodes As Collection
Private toCSCanMedCarrierCodes As Collection


Public Function Init() As Boolean

    'Receives: Nothing.
    'Returns: True if the carrier translation database exists.  Otherwise, False.
    'Verify that db exists, read data from it into forward and reverse translation collections.
    
    carToCsDb = ProjectDir() & "cartocs.dbf"
    If fExist(carToCsDb) Then
        Init = PopulateCarrTransCollections()
    Else
        Dim errorMessage As String
        errorMessage = "Carrier Translation table: " & carToCsDb & " does not exist." & vbCrLf
        defltLog.PopAndLog errorMessage, "Error - CarrierTranslation.Init"
        Init = False
    End If

End Function

Public Function ToCS(carrier As String, Optional pack As String = "") As String
    
    'Receives: String containing the host carrier code.
    '          String containing the tax region field from the host.
    'Returns: Nothing
    'If the TaxRegion field contains the text "USA" then the carrier will be changed to OTHU.
    'If the TaxRegion field contains the text "Intl" then the carrier will be changed to OTHI.
    'Attempts to locate the host carrier code in the carrierCodes Collection.
    'If a carrierCode instance is found, the CarrierTranslation properties are populated with the appropriate data from the instance.

    Dim foundCode As String
    
    If pack = "USA" Then
        foundCode = "OTHU"
    ElseIf pack = "Intl" Then
        foundCode = "OTHI"
    Else
        If cs_ProcMode = cs_OOSMode Then
            foundCode = LookupInColl(Upper(carrier), toCSOOSCarrierCodes)
        Else
            foundCode = LookupInColl(Upper(carrier), toCSCanMedCarrierCodes)
        End If
    End If
    
    If Len(foundCode) > 0 Then
        ToCS = foundCode
    Else
        defltLog.PopAndLog "Unable to find host carrier code " & carrier & " in carrier translation.", "Error - CarrierTranslation.ToCS"
        ToCS = defaultHostCarrierCode
    End If
   
End Function
    
Public Sub EditDatabase()

  'Receives: Nothing.
  'Returns: Nothing.
  'Packs and indexes database, allows user to change values.

  DbfIndex carToCsDb, "HOSTCARR"
  DbfCleanup carToCsDb
  DbfEdit carToCsDb
  DbfCleanup carToCsDb
  
  MsgBox "Please restart all instances of Clippership to ensure that they are using the latest carrier translations.", , "Warning - CarrierTranslation.EditDatabase"
  
End Sub

Private Function PopulateCarrTransCollections() As Boolean

    'Receives: Nothing
    'Returns: True if successfully reads in all records, and there is at least one, False otherwise.
    'Populates the collection of CarrierCodes with the values from CARTOCS.DBF
    
    'Retrieve all of the carriers from the Carrier translation database.
    'For each record, create two instances of CarrierCode.  One for the translation to Clippership and one
    'for the translation to the host.
    Dim counter As Integer
    Dim myDV As KiDataView
    Set myDV = DbfSelect(carToCsDb, "1=1", , 0)
    myDV.MoveFirst
    counter = 0
    Set toCSCanMedCarrierCodes = New Collection
    Set toCSOOSCarrierCodes = New Collection
    
    'It is okay for the Clippership carrier to be a duplicate since the carrier code that we read in will be the carrier
    'code that we write back.  For rate shop, they don't care which carrier code is written back, as long as the carrier and service
    'are correct.
    'There is no need to catch an error when adding to the collection.
    
On Error GoTo ReadingError
    
    Dim errColl As String
    Dim hostCarr As String
    Dim csCarr As String
    Do While Not myDV.EOF
        hostCarr = Trim(Upper(myDV("HOSTCARR")))
        csCarr = Trim(Upper(myDV("CSCARR")))
        
        If Len(hostCarr) > 0 And Len(csCarr) > 0 Then
            If Trim(myDV("MODE")) = "OOS" Then
                errColl = "ToCS OOS"
                toCSOOSCarrierCodes.Add csCarr, hostCarr
            ElseIf Trim(myDV("MODE")) = "CANMED" Then
                errColl = "ToCS CanMed"
                toCSCanMedCarrierCodes.Add csCarr, hostCarr
            End If
        End If
        myDV.MoveNext
        counter = counter + 1
    Loop
    GoTo Finish
    
ReadingError:
    Dim errMsg As String
    errMsg = "Hit error when trying to add csCarr " & csCarr & " and hostCarr " & hostCarr & " to collection " & errColl
    errMsg = errMsg & " with count=" & CStr(counter)
    defltLog.PopAndLog errMsg, "Error - CarrierTranslation.PopulateCarrTransCollections"
    counter = 0
    
Finish:
    Set myDV = Nothing
    If counter > 0 Then
        PopulateCarrTransCollections = True
    Else
        PopulateCarrTransCollections = False
    End If
    
End Function

Private Function LookupInColl(lookupCode As String, lookupColl As Collection) As String
    
    'Receives: String containing the carrier code to look up, the collection to look it up in.
    'Returns: String containing carrier code we found, or empty string if nothing found.
    'Attempts to locate the given carrier code in the given Collection.

On Error GoTo NotFound
    
        Dim returnCode As String
        returnCode = lookupColl(Trim(Upper(lookupCode)))
        GoTo Finish
    
NotFound:
        returnCode = ""

Finish:
        LookupInColl = returnCode
    
End Function

Public Sub RunTests()

    'Receives: Nothing.
    'Returns: Nothing.
    'Runs some tests on carrier translation, to test lookups.
    
    Dim numFailures As Integer
    numFailures = 0
    
    'ToCs tests.
    If Not TestToCs("68", "MCE") Then
        numFailures = numFailures + 1
    End If
    
    If Not TestToCs("81", "USM") Then
        numFailures = numFailures + 1
    End If
    
    If Not TestToCs("92", "UEX") Then
        numFailures = numFailures + 1
    End If
    
    If Not TestToCs("40", "USK") Then
        numFailures = numFailures + 1
    End If
    
    If Not TestToCs("84", "OTHR") Then
        numFailures = numFailures + 1
    End If
    
    If TestToCs("68", "MCR") Then
        numFailures = numFailures + 1
    End If
    
   MsgBox "Number of failures=" & CStr(numFailures), , "CarrierTranslation.RunTests"
    
End Sub

Private Function TestToCs(hostCar As String, csCar As String) As Boolean

    'Receives: Host carrier code, CS carrier code.
    'Returns: True if test succeeds, False otherwise.
    'Verifies that gets expected result from ToCs function. Displays error if fails.
    
    Dim foundCar As String
    
    foundCar = ToCS(hostCar)
    If foundCar = csCar Then
        TestToCs = True
    Else
        MsgBox "For host carrier " & hostCar & " expected to find " & csCar & " but actually found " & foundCar, , "TestToCS"
        TestToCs = False
    End If
    
End Function

