Attribute VB_Name = "HandlingCharge"
Option Explicit

'-------------------------------------------------------------------------------------------------------
'Custom!!! - The freight mark-up charge is based on the destination province.
'OOS will have a self-maintained table that will allow them to enter a percentage by province code.
'If the province code is not found, mark-up charge will not be calculated.
'This is just an estimate of the taxes and we are supposed to apply it to the total shipping charges,
'including everything - base amount, accessories, and surcharges.
'-------------------------------------------------------------------------------------------------------

Private hndRatesDb As String                            'Path and name of handling charge database.
Private chgFields() As String                           'Clippership field names to calculate the handling charge on.
                                                        'whether to apply the minimum charge.
Private Const provinceNotFoundAction As String = "1"    '1 - Display warning, allow to continue.
                                                        '2 - Display error, prevent from shipping.

Public Function Init() As Boolean

  'Receives: Nothing.
  'Returns: True if handling charge file exists, false otherwise.
  'Initializes properties.
  
  ReDim chgFields(0)
  chgFields(0) = "TOT_CHG"                      'Array of fields that want to base variable handling charge on. Note that should
                                                'never use shp_chg without also including sur_chg (unless want to base it on
                                                'non-discounted rate). Beyond charges are generally in pod_chg, for .Net carriers.
  Dim fileExists As Boolean
  hndRatesDb = ProjectDir() & "hndrates.dbf"    'Database that contains handling charge numbers.
  If fExist(hndRatesDb) Then
    DbfIndex hndRatesDb, "PROV"
    fileExists = True
  Else
    defltLog.PopAndLog "Unable to find handling charge file " & hndRatesDb, "Error-HandlingCharge.Init"
    fileExists = False
  End If
  
  Init = fileExists
  
End Function

Public Sub Calculate(myXVars As KiDataView)

  'Receives: KiDataView containing xvars for current package.
  'Returns: Nothing.
  'Calculates handling charge for current package, stores it in hnd_chg, adds it to tot_chg, ohnd_chg and otot_chg.

  'Look up current province code in database, get handling charge parameters.
  Dim hndDv As KiDataView
  Set hndDv = GetHandlingRates(myXVars("SHIPTO7"))
  If Not (hndDv Is Nothing) Then
    'Found a record, so calculate the charges.
    Dim handling As Currency
    Dim numRecs As Integer
    numRecs = GetNumRecs(myXVars)
      
    'Variable handling charge
    handling = GetVariableHandlingChg(myXVars, Val(hndDv("VAR_RATE")))
  Else
    'Default handling charge when province is not found in the database
    handling = gDefltHandRate
  End If
  
  'Send handling charge back to Clippership.
  cshost.SetVar "XHND_CHG", handling
  cshost.SetVar "XTOT_CHG", myXVars("TOT_CHG") + handling
  cshost.SetVar "XOHND_CHG", myXVars("OHND_CHG") + handling
  cshost.SetVar "XOTOT_CHG", myXVars("OTOT_CHG") + handling
    
End Sub

Public Sub Maintenance()

  'Receives: Nothing.
  'Returns: Nothing.
  'Packs and indexes database, allows user to change values.
  
  DbfIndex hndRatesDb, "PROV"
  DbfCleanup hndRatesDb
  DbfEdit hndRatesDb
  DbfCleanup hndRatesDb

End Sub

Private Function GetHandlingRates(myProvince As String) As KiDataView

  'Receives: Province.
  'Returns: KiDataView containing handling rates.
  'See if can find given Province in handling rate database. If find one record, return it, otherwise give error, return nothing.
  
  Dim hndDv As KiDataView
  Set hndDv = DbfLookup(hndRatesDb, PadR(Trim(myProvince), 10), True, , 0)
  If hndDv Is Nothing Then
    'Must have lost database, or connection to it - will return nothing.
    GetHandlingRatesError "Unable to access handling charge database - DbfLookup returned nothing."
  ElseIf hndDv.RowCount = 1 Then
    'Found a record, so can calculate the charges.
  ElseIf hndDv.RowCount = 0 Then
    'Current province isn't in table.
    'GetHandlingRatesError "Unable to find Province " & Trim(myProvince) & " in handling charge database."
    'If not found in the database, use the default handling charge
    Set hndDv = Nothing
  Else
    'Current Province is in table multiple times - give error, don't calculate charge.
    Dim myMsg2 As String
    myMsg2 = "Found " & CStr(hndDv.RowCount) & " records for " & myProvince & " in handling charge database."
    myMsg2 = myMsg2 & " Not calculating handling charge"
    defltLog.PopAndLog myMsg2, "Error-HandlingCharge.GetHandlingRates"
    Set hndDv = Nothing
  End If
    
  Set GetHandlingRates = hndDv
  
End Function

Private Sub GetHandlingRatesError(myMsg As String)

  'Receives: String containing error message.
  'Returns: Nothing.
  'Displays or displays and logs warning, as appropriate.
  
  If provinceNotFoundAction = "1" Then
    'Display warning but allow user to continue.
    defltLog.PopAndLog myMsg, "Error-HandlingCharge.GetHandlingRates"
  ElseIf provinceNotFoundAction = "2" Then
    'Display warning and prevent user from recording package.
    cshost.SetError myMsg
  Else
    'Display 2 warnings - provinceNotFoundAction improperly set, and province not configured.
    defltLog.PopAndLog "provinceNotFoundAction is set to an invalid value: " & provinceNotFoundAction, "Error-HandlingCharge.GetHandlingRates"
    defltLog.PopAndLog myMsg, "Error-HandlingCharge.GetHandlingRates"
  End If

End Sub

Private Function GetVariableHandlingChg(myXVars As KiDataView, varRate As Currency) As Currency

  'Receives: KiDataView containing xVars for current package, rate for variable handling charge.
  'Returns: The variable handling charge for the current package.
  'Calculate the variable handling charge for this piece, including minimum.
    
  Dim tmpVarHnd As Currency
  tmpVarHnd = Round(GetBaseFreight(myXVars) * (varRate / 100), 2)
  GetVariableHandlingChg = tmpVarHnd
  
End Function

Private Function GetNumRecs(myXVars As KiDataView) As Integer

  'Receives: KiDataView containing xVars for current package.
  'Returns: Number of records that will be put in the ship file for this order.
  'Figure out how many records will end up in the ship file for this order.
  'This function assumes that IPD and TD will only be used from a Canadian origin, and that, if used, they result in one
  'record being stored in the ship file for each package.
  'This function also assumes that we only ship from Canadian or US origins - i.e. if val(originzip) > 0, then we are
  'shipping from the US, otherwise are shipping from Canada.
  
  Dim numRecs As Integer
  Dim intcode As String
  intcode = Left(myXVars("INTCODE"), 1)
'Should be using isFedex
  'Fedex Grd and Express occasionally store data in funny ways, so need to do extra checks for them.
  If intcode = "R" Or intcode = "F" Then
    'If domestic origin, and not IPD or TD, then should be using the mps file, and thus have just one record.
    If Val(Trim(cshost.GetVar("XFROMZIP"))) > 0 Then
      'Shipping from US origin, always one record per package.
      numRecs = Val(myXVars("EXP_CNT"))
    ElseIf myXVars("VDC_FLAG") = "Y" Or Len(Trim(cshost.GetVar("XFDXIOR"))) > 0 Then
      'vdc_flag=Y means TD (for fedex pkgs), fdxior should only be set for IPD.
      numRecs = Val(myXVars("EXP_CNT"))
    Else
      numRecs = 1
    End If
  Else
    'Not fedex express/grd, so should be exp_cnt records in the ship file.
    numRecs = Val(myXVars("EXP_CNT"))
  End If
  
  GetNumRecs = numRecs
  
End Function

Private Function GetBaseFreight(myXVars As KiDataView) As Currency

  'Receives: KiDataView containing xVars for current package.
  'Returns: The "base freight" that the customer wants to base the variable handling charge on.
  'Calculate the base freight for the current package, by iterating through the fields in the chgFields array.
  
  Dim baseFreight As Currency
  Dim i As Integer
  baseFreight = 0
  For i = 0 To UBound(chgFields)
    baseFreight = baseFreight + Val(myXVars(chgFields(i)))
  Next i
  
  GetBaseFreight = baseFreight
  
End Function
