Attribute VB_Name = "VanguardEDI"
'Vanguard EDI Module V2.00 March 22, 2016
'Assumptions:
'1. Will only be one Vanguard account (i.e. project id) per system.
'2. LTL carriers are always set up with one tracking number per shipment.
'3. There are no generic couriers used with Vanguard.
'4. Pkgid's are unique within a daily file.

Option Explicit

Public vgCarTrans       As Collection               'Collection of CS carrier/VG carrier pairs, CS carrier is key.
Public ediRef1Field     As String                   'Name of CS field that want to put in shipper reference field in edi file.
Public ediRef2Field     As String                   'Name of second CS field that want to put in shipper reference field in edi file.
Public projectID        As String                   'Project Identifier assigned by Vanguard to this customer.
Private serviceList     As String                   'Comma-delimited list of carrier codes associated with Vanguard. Must end with ,.
Private vgUserConfig    As String                   'Full path and name of ini file containing ftp user/pass, plus system-specific VG info
Private ftpHostConfig   As String                   'Full path and name of ini file containing ftp site/port.
Private ediFilePath     As String                   'Full path to directory containing edi files, ending in \.
Private ediBakPath      As String                   'Full path to directory containing backup edi files, ending in \.
Private ediFileExt      As String                   'Extension for edi files - i.e. ".csv"
Private myVanguardRecord As VanguardRecord
Private currEdiFileName As String                   'Set before calling edi file generator. Contains just file name, not path.
Private Const defaultFreightClass As String = "70"  'Default LTL Freight Class value

Public Sub Init()
    
    'Receives: Nothing.
    'Returns: Nothing.
    'Populates all properties.
    
    'Config files are both in the KI directory.
    Dim currProjectDir As String
    currProjectDir = ProjectDir()
    
    vgUserConfig = currProjectDir & "vgcfg.ini"
    ftpHostConfig = currProjectDir & "vgftp.ini"
    
    'Verify that both files exist, before reading anything.
    If fExist(vgUserConfig) Then
        If fExist(ftpHostConfig) Then
            projectID = IniRead(vgUserConfig, "Vanguard", "ProjectId")
            serviceList = IniRead(vgUserConfig, "Vanguard", "Services") & ","
            ediRef1Field = IniRead(vgUserConfig, "Vanguard", "EdiRef1Field", "PROD")
            ediRef2Field = IniRead(vgUserConfig, "Vanguard", "EdiRef2Field", "")
            ediFilePath = cshost.ShipDir & "evcanedi\vanguard\"
            ediFileExt = ".csv"
            
            'Call routine to populate the carrier translation collections.
            PopCarTrans
            
            'Verify that edi and edi backup directories exist. Unfortunately, if just try to create the bak dir,
            'if the parent dir doesn't exist, will get a KI error. Thus, must create both.
            If Not fExist(ediFilePath, vbDirectory) Then
                MkDir ediFilePath
            End If
            ediBakPath = ediFilePath & "bak\"
            If Not fExist(ediBakPath, vbDirectory) Then
                MkDir ediBakPath
            End If
        Else
            MsgBox "Vanguard ftp config file " & ftpHostConfig & " is missing.", , "VanguardEdi.Init"
        End If
    Else
        MsgBox "Vanguard config file " & vgUserConfig & " is missing.", , "VanguardEdi.Init"
    End If
        
End Sub

Public Sub Maintenance()

    'Receives: Nothing.
    'Returns: Nothing.
    'Removes old files from edi backup dir.
    
    Dim numDays As Integer
    numDays = Val(IniRead(vgUserConfig, "Vanguard", "EdiDays", "200"))
    
    Utils.RemoveOldFiles ediBakPath, numDays
    
End Sub

Public Function IsVanguardLTL(csCarrier As String) As Boolean

    'Receives: Clippership carrier code.
    'Returns: True if given carrier is associated with Vanguard and is LTL, False otherwise.
    'Checks whether given carrier is in the service list and if it is LTL.
    
    If IsVanguardService(csCarrier) And Utils.IsLTL(csCarrier) Then
        IsVanguardLTL = True
    Else
        IsVanguardLTL = False
    End If

End Function

Public Sub PopulateVanguardClass(carrier As String, myClass As String, zip As String)

    'Receives: Strings containing the Clippership carrier code, existing value for the LTL class, and the postal code.
    'Returns: Nothing
    'Defaults the value of the CLASS field to "70" for Vanguard LTL shipments going to the US unless the field
    'is already populated.
    
    Dim vanguardClass As String
    
    If IsVanguardService(carrier) And Utils.IsLTL(carrier) And Len(Trim(myClass)) = 0 And Val(zip) > 0 Then
        vanguardClass = defaultFreightClass
    Else
        vanguardClass = myClass
    End If
    
    cshost.SetVar "xclass", vanguardClass

End Sub

Public Sub PopLTLBillto(csCarrier As String, csPayFlag As String)

    'Receives: Carrier, pay_flag
    'Returns: Nothing
    'Populate billto address if Vanguard LTL and prepaid.

    If VanguardEDI.IsVanguardLTL(csCarrier) And csPayFlag = "1" Then
        cshost.SetVar "xBillTo1", ""
        cshost.SetVar "xBillTo2", ""
        cshost.SetVar "xBillTo3", ""
        cshost.SetVar "xBillTo4", ""
        cshost.SetVar "xBillTo5", "VANGUARD GLOBAL SERVICES"
        cshost.SetVar "xBillTo6", "265 EXPORT BLVD"
        cshost.SetVar "xBillTo7", "MISSISSAUGA ON, L5S 1Y4"
    End If

End Sub

Public Sub CreateAndSend(retransmit As String)

    'Receives: retransmit flag to indicate if to ignore the already written back to host flag when creating the EDI file
    'Returns : Nothing
    'Called from Clippership.TransmitVanguardEDI()
    'Manages process of creating edi file for given daily file, and transmitting all outstanding edi files.
    
    PopStatus "Please Wait...", "Creating Vanguard EDI File.", True
    currEdiFileName = GetNextFileName()
    
    Set myVanguardRecord = New VanguardRecord
    Utils.ProcessDailyFile retransmit, gVangEDIProcFlagField, "VANGUARD", "pkgid"
    KillStatus
    
    'Copy file that was just created to backup dir.
    Dim srcFile As String
    Dim destFile As String
    srcFile = ediFilePath & currEdiFileName
    destFile = ediBakPath & currEdiFileName
    If fExist(srcFile) Then
        fCopy srcFile, destFile
    End If
    
    'Am calling transmit regardless of whether a file was just created, in case prior files haven't been transmitted.
    KillStatus
    Utils.SendFtp vgUserConfig, ftpHostConfig, ediFilePath, "*" & ediFileExt

End Sub

Public Sub ProcessRecords()

    'Receives: Nothing
    'Returns : Nothing
    'Called from Utils.ProcessDailyFile()- has to be public as it is used by a DbfProcess() function
    'Creates a package level record in the EDI file for each record in the daily file if courier, one record
    'per order if ltl.
    'Caller doesn't filter out non-vanguard carriers, so this function must do it.
    
    If IsVanguardService(CStr(XV1("CARRIER"))) Then
        'Have to copy XV1 to a KiDataView, as XV1 is a variant.
        Dim tmpDV As KiDataView
        Set tmpDV = XV1
        
        'Only initialize the totals for every courier package and for the first package / skid in an LTL order.
        If Not IsLTL(CStr(tmpDV("CARRIER"))) Or Val(tmpDV("ORD_CNT")) = 1 Then
            myVanguardRecord.InitializeTotals
        End If
        
        myVanguardRecord.SetRecordData tmpDV
        
        Dim myRecord As String
        myRecord = myVanguardRecord.GetRecordData(tmpDV)
        If Len(Trim(myRecord)) > 0 Then
            Host2.AppendLineTo ediFilePath & currEdiFileName, myRecord
        End If
        
        ' Set the transmitted flag to ensure that this package does not get retransmitted.
        ' Needs to be set even if the record is not created in order to be able to handle
        ' LTL orders with multiple skids / pieces.
        DbfReplace gVangEDIProcFlagField, "Y"
    End If
        
End Sub

Private Function GetNextFileName() As String

    'Receives: Nothing
    'Returns: Name of next edi file to create, including path.
    'Generate next edi file name.
    'EDI file name = project id + "_" + 5 digit number + ".csv".
    
    ' Read in the Vanguard EDI file number
    Dim ediCtrFile As String
    Dim ediNumber As Long
    
    ediCtrFile = cshost.ShipDir() & "vanguard.ini"
    ediNumber = Val(IniRead(ediCtrFile, "EDI", "EDIcounter", "1"))
    If ediNumber < 1 Then
        ediNumber = 1
    ElseIf ediNumber > 99999 Then
        ediNumber = 1
    Else
        ediNumber = ediNumber + 1
    End If
    IniWrite ediCtrFile, "EDI", "EDIcounter", Trim(Str(ediNumber))
    
    GetNextFileName = projectID & "_" + PadL(Alltrim(Str(ediNumber)), 5, "0") & ediFileExt

End Function

Private Sub PopCarTrans()

    'Receives: Nothing.
    'Returns: Nothing.
    'Populates the carrier translation collections, based on contents of serviceList and the vgTrans database.
    
    Dim vgCarTransDbf As String
    vgCarTransDbf = ProjectDir() & "vgtrans.dbf"
    If fExist(vgCarTransDbf) Then
        'Verify that index exists and is up to date.
        Utils.VerifyIndex vgCarTransDbf, "cscarrier"
        
        'Split serviceList into array, each element containing one clippership carrier code. Note that
        'the last element will be empty, because of trailing comma. For each element in array, look it up
        'in translation database, and add vgcarrier/service to the appropriate collection. If not found,
        'emit error - need to be able to translate every vanguard carrier on the system.
        Dim servArr As Variant
        Dim serv As Variant         'Argument in "for each" has to be a variant - can't be a string.
        Dim servStr As String
        Dim myRec As KiDataView
        servArr = Split(serviceList, ",")
        
        Set vgCarTrans = New Collection
        
        For Each serv In servArr
            servStr = Upper(Trim(CStr(serv)))
            If Len(servStr) > 0 Then
                Set myRec = DbfLookup(vgCarTransDbf, PadR(servStr, 10))
                If Not myRec Is Nothing Then
                    If myRec.RowCount > 0 Then
                        vgCarTrans.Add Trim(myRec("vgcarrier")), servStr
                    Else
                        MsgBox "Unable to find record for " & servStr & " in translation database " & vgCarTransDbf, , "VanguardEdi.PopCarTrans"
                    End If
                Else
                    MsgBox "Unable to find record for " & servStr & " in translation database " & vgCarTransDbf, , "VanguardEdi.PopCarTrans"
                End If
            End If
        Next
        Set myRec = Nothing
    Else
        MsgBox "Error - Vanguard carrier translation database " & vgCarTransDbf & " is missing.", , "VanguardEdi.PopCarTrans"
    End If
    
End Sub

Private Function IsVanguardService(csCarrier As String) As Boolean

    'Receives: Clippership carrier code.
    'Returns: True if given carrier is associated with Vanguard, False otherwise.
    'Checks whether given carrier is in the service list.
    
    If InStr(serviceList, Trim(Upper(csCarrier)) & ",") > 0 Then
        IsVanguardService = True
    Else
        IsVanguardService = False
    End If
    
End Function

